@@ -30,7 +30,9 @@ class Api(snappi.Api):
             host='https://127.0.0.1:11009' if host is None else host
         )
         self._c = STLClient()
-        self._portIndices = []
+        self._port_ids = []
+        self._captures = []
+        self._transmit_state = 'stop'
         try:
             # connect to server
             self._c.connect()
@@ -51,7 +53,7 @@ class Api(snappi.Api):
         if 'ports' in self._cfg:
             i = 0
             for p in self._cfg['ports']:
-                self._portIndices.append(p['name'])
+                self._port_ids.append(p['name'])
 
 
     def set_config(self, config):
@@ -66,11 +68,10 @@ class Api(snappi.Api):
 
         try:
             # prepare our ports
-            self._c.reset(ports = list(range(len(self._portIndices))))
+            self._c.reset(ports = list(range(len(self._port_ids))))
 
             # for each Snappi flow, construct the equivalent T-Rex stream
             for f in self._cfg["flows"]:
-                
                 # Configure variable manager commands
                 vmCmds = []
 
@@ -107,7 +108,7 @@ class Api(snappi.Api):
                             mode = mode)
 
                 # Add the stream to the client
-                self._c.add_streams([s1], ports=[self._portIndices.index(f['tx_rx']['port']['tx_name'])])
+                self._c.add_streams([s1], ports=[self._port_ids.index(f['tx_rx']['port']['tx_name'])])
 
         # Disconnect on error
         except STLError as e:
@@ -121,16 +122,26 @@ class Api(snappi.Api):
     def set_transmit_state(self, payload):
         """Set the transmit state of flows
         """
+        ts = json.loads(payload.serialize())
+        ports = list(range(len(self._port_ids)))
+        if ts['flow_names'] is not None:
+            ports = []
+            for f_name in ts['flow_names']:
+                ports.append(self._flow_ids.index(f_name))
         try:
-            # Clear stats
-            self._c.clear_stats()
-
-            # Start streams on all ports
-            self._c.start(ports = list(range(len(self._portIndices))))
-
-            # Wait until traffic stops
-            self._c.wait_on_traffic(ports = list(range(len(self._portIndices))))
-
+            if ts['state'] == 'start':
+                if self._transmit_state == 'stop':
+                    self._c.clear_stats()
+                    self._c.start(ports = ports)
+                elif self._transmit_state == 'pause':
+                    self._c.resume(ports = ports)
+            elif ts['state'] == 'stop':
+                self._c.stop(ports = ports)
+            elif ts['state'] == 'pause':
+                self._c.pause(ports = ports)
+            
+            self._transmit_state = ts['state']
+            self._c.wait_on_traffic(ports = ports)
         except STLError as e:
             self._c.disconnect()
             print(e)
@@ -139,8 +150,19 @@ class Api(snappi.Api):
     def set_capture_state(self, payload):
         """Starts capture on all ports that have capture enabled.
         """
+        cs = json.loads(payload.serialize())
+        ports = list(range(len(self._port_ids)))
+        if cs['port_names'] is not None:
+            ports = []
+            for f_name in cs['port_names']:
+                ports.append(self._port_ids.index(f_name))
         try:
-            """"""
+            if cs['state'] == 'start':
+                self._captures.append(self._c.start_capture(rx_ports = ports))
+            elif cs['state'] == 'stop':
+                while len(self._captures) > 0:
+                    cap = self._captures.pop(0)
+                    self._c.stop_capture(cap['id'], "/home/frederick/cap/cap_{}.pcap".format(cap['id']))
         except STLError as e:
             self._c.disconnect()
             print(e)
